import { select, transition, easeLinear } from 'd3';

export const main = (container) => {
  const width = container.clientWidth;
  const height = container.clientHeight;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height)
    .style('background', '#000000');

  const data = [
    {
      name: 'Sun',
      distance: 0,
      radius: 109,
      fill: '#FFC800',
    },
    {
      name: 'Mercury',
      distance: 0.38709893,
      radius: 0.3826,
      fill: '#969696',
      labelOffset: -76,
    },
    {
      name: 'Venus',
      distance: 0.72333199,
      radius: 0.9488,
      fill: '#DDD3C2',
      labelOffset: -61,
    },
    {
      name: 'Earth',
      distance: 1,
      radius: 1,
      fill: '#6388B4',
      labelOffset: -45,
    },
    {
      name: 'Mars',
      distance: 1.52366231,
      radius: 0.53247,
      fill: '#C1440E',
      labelOffset: -27,
    },
    {
      name: 'Jupiter',
      distance: 5.20336301,
      radius: 11.209,
      fill: '#E5b380',
    },
    {
      name: 'Saturn',
      distance: 9.53707032,
      radius: 9.449,
      fill: '#D9BF77',
    },
    {
      name: 'Uranus',
      distance: 19.19126393,
      radius: 4.007,
      fill: '#8FD3FF',
    },
    {
      name: 'Neptune',
      distance: 30.06896348,
      radius: 3.883,
      fill: '#4385F5',
    },
    {
      name: 'Pluto',
      distance: 39.482,
      radius: 0.1868,
      fill: '#E3DBCF',
    },
  ];

  const slow = transition().duration(2000);
  const fast = transition().duration(500).ease(easeLinear);

  const xValue = (d) =>
    d.name === 'Sun'
      ? -325
      : (d.distance * 2217) / 100 + 47;
  const yValue = height / 2;
  const rValue = (d) => (d.radius * 322) / 100;

  const labelValue = (d) => d.name;
  const xLabelValue = (d) =>
    d.name === 'Sun' ? 20 : (d.distance * 2217) / 100 + 47;
  const yLabelValue = (d) =>
    d.name === 'Sun'
      ? 400
      : rValue(d) +
        yValue +
        5 +
        (d.labelOffset ? d.labelOffset : 0);

  // Planets
  svg
    .selectAll('circle')
    .data(data, (d) => d.name) // key function
    .join(
      (enter) =>
        enter
          .append('circle')
          .attr('cx', xValue)
          .attr('cy', yValue)
          .attr('r', 0)
          .call((selection) => {
            selection
              .transition(slow)
              .delay((d, i) => i * 1000)
              .attr('r', rValue);
          }),
      (update) =>
        update.call((selection) => {
          selection
            .transition(fast)
            .attr('cx', xValue)
            .attr('cy', yValue)
            .attr('r', rValue);
        }),
      (exit) => exit.transition(slow).attr('r', 0).remove(),
    )
    .attr('fill', (d) => d.fill)
    .attr('opacity', 708 / 1000);

  // Labels
  svg
    .selectAll('text')
    .data(data, (d) => d.name) // key function
    .join(
      (enter) =>
        enter
          .append('text')
          .attr('x', xLabelValue)
          .attr('y', yLabelValue)
          .attr('opacity', 0)
          .text(labelValue)
          .call((selection) => {
            selection
              .transition(slow)
              .delay((d, i) => i * 1000)
              .attr('opacity', 1);
          }),
      (update) =>
        update.call((selection) => {
          selection
            .transition(fast)
            .attr('x', xLabelValue)
            .attr('y', yLabelValue);
        }),
      (exit) =>
        exit.transition(slow).attr('opacity', 0).remove(),
    )
    .attr('fill', 'white')
    .style('font-family', 'sans-serif')
    .style('font-size', 14)
    .attr('text-anchor', 'middle')
    .attr('alignment-baseline', 'hanging');
};
